﻿using System.Transactions;
using Bunny.Common.Attribute;
using Bunny.Common.Context;
using log4net;
using Microsoft.AspNetCore.Mvc.Controllers;
using Microsoft.AspNetCore.Mvc.Filters;

namespace Bunny.Common.Filter;

public class TransactionIAsyncActionFilter : IAsyncActionFilter
{
    private static readonly ILog Log = LogManager.GetLogger(typeof(TransactionIAsyncActionFilter));

    public async Task OnActionExecutionAsync(ActionExecutingContext context, ActionExecutionDelegate next)
    {
        var actionDesc = context.ActionDescriptor as ControllerActionDescriptor;
        var hasNotTransactionalAttribute =
            actionDesc?.MethodInfo.IsDefined(typeof(TransactionAttribute), false) ?? false;

        if (hasNotTransactionalAttribute)
        {
            await next();
            return;
        }

        using var txScope = new TransactionScope(TransactionScopeAsyncFlowOption.Enabled);
        var result = await next();
        if (result.Exception == null)
        {
            txScope.Complete();
        }
        else
        {
            var errorMessage =
                $"执行数据库出错 用户Id：[{BaseContext.GetUserId()}] token [{BaseContext.GetToken()}]:{result.Exception}";
            Log.Error(errorMessage);
        }
    }
}